# -*- coding: utf-8 -*-
#
# This file is part of INSPIRE.
# Copyright (C) 2014-2017 CERN.
#
# INSPIRE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# INSPIRE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with INSPIRE. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Convert incoming MARCXML to JSON."""

from __future__ import absolute_import, division, print_function

from inspire_utils.helpers import force_list

from .conferences import conferences
from .data import data
from .experiments import experiments
from .hep import hep
from .hepnames import hepnames
from .institutions import institutions
from .jobs import jobs
from .journals import journals


def overdo_marc_dict(record):
    """Convert MARC Groupable Ordered Dict into JSON."""
    if _collection_in_record(record, 'institution'):
        return institutions.do(record)
    elif _collection_in_record(record, 'experiment'):
        return experiments.do(record)
    elif _collection_in_record(record, 'journals') or \
            _collection_in_record(record, 'journalsnew'):
        return journals.do(record)
    elif _collection_in_record(record, 'hepnames'):
        return hepnames.do(record)
    elif _collection_in_record(record, 'job') or \
            _collection_in_record(record, 'jobhidden'):
        return jobs.do(record)
    elif _collection_in_record(record, 'conferences'):
        return conferences.do(record)
    elif _collection_in_record(record, 'data'):
        return data.do(record)
    else:
        return hep.do(record)


def _collection_in_record(record, collection):
    """Returns True if record is in collection"""
    colls = force_list(record.get("980__", []))
    for coll in colls:
        coll = force_list(coll.get('a', []))
        if collection in [c.lower() for c in coll]:
            return True
    return False



# -*- coding: utf-8 -*-
#
# This file is part of INSPIRE.
# Copyright (C) 2014-2017 CERN.
#
# INSPIRE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# INSPIRE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with INSPIRE. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""DoJSON model definition for conferences."""

from __future__ import absolute_import, division, print_function

from ..model import FilterOverdo, add_schema, add_collection, clean_record


def remove_lone_series_number(record, blob):
    def _valid(series):
        return series.get('name')

    record['series'] = list(filter(_valid, record.get('series', [])))

    return record


filters = [
    add_schema('conferences.json'),
    add_collection('Conferences'),
    remove_lone_series_number,
    clean_record,
]

conferences = FilterOverdo(filters=filters)



# -*- coding: utf-8 -*-
#
# This file is part of INSPIRE.
# Copyright (C) 2014-2017 CERN.
#
# INSPIRE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# INSPIRE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with INSPIRE. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""DoJSON rules for jobs."""

from __future__ import absolute_import, division, print_function

import re

import six

from dojson import utils

from inspire_utils.helpers import force_list

from .model import jobs
from ..utils import (
    force_single_element,
    get_record_ref,
    normalize_rank,
    normalize_date_aggressively,
)


COMMA_OR_SLASH = re.compile('\s*[/,]\s*')


@jobs.over('closed_date', '^046..')
def date_closed(self, key, value):
    def _contains_email(val):
        return '@' in val

    def _contains_url(val):
        return 'www' in val or 'http' in val

    el = force_single_element(value)

    deadline_date = force_single_element(el.get('i'))
    if deadline_date:
        self['deadline_date'] = normalize_date_aggressively(deadline_date)

    closing_date = None
    closing_info = force_single_element(el.get('l'))
    if closing_info:
        if _contains_email(closing_info):
            if 'reference_email' in self:
                self['reference_email'].append(closing_info)
            else:
                self['reference_email'] = [closing_info]
        elif _contains_url(closing_info):
            if 'urls' in self:
                self['urls'].append({'value': closing_info})
            else:
                self['urls'] = [{'value': closing_info}]
        else:
            closing_date = normalize_date_aggressively(closing_info)

    return closing_date


@jobs.over('contact_details', '^270..')
@utils.for_each_value
def contact_details(self, key, value):
    name = value.get('p')
    email = value.get('m')

    return {
        'name': name if isinstance(name, six.string_types) else None,
        'email': email if isinstance(email, six.string_types) else None,
    }


@jobs.over('regions', '^043..')
def regions(self, key, value):
    REGIONS_MAP = {
        'AF': 'Africa',
        'Africa': 'Africa',
        'Asia': 'Asia',
        'Australia': 'Australasia',
        'Australasia': 'Australasia',
        'eu': 'Europe',
        'Europe': 'Europe',
        'Middle East': 'Middle East',
        'na': 'North America',
        'United States': 'North America',
        'Noth America': 'North America',
        'North America': 'North America',
        'North Americsa': 'North America',
        'South America': 'South America',
    }

    result = []

    for el in force_list(value.get('a')):
        for region in COMMA_OR_SLASH.split(el):
            result.append(REGIONS_MAP.get(region))

    return result


@jobs.over('experiments', '^693..')
def experiments(self, key, value):
    experiments = self.get('experiments', [])

    name = value.get('e')
    recid = value.get('0')
    record = get_record_ref(recid, 'experiments')

    experiments.append({
        'curated_relation': record is not None,
        'name': name,
        'record': record
    })

    return experiments


@jobs.over('institutions', '^110..')
def institutions(self, key, value):
    institutions = self.get('institutions', [])

    a_values = force_list(value.get('a'))
    z_values = force_list(value.get('z'))

    # XXX: we zip only when they have the same length, otherwise
    #      we might match a value with the wrong recid.
    if len(a_values) == len(z_values):
        for a_value, z_value in zip(a_values, z_values):
            record = get_record_ref(z_value, 'institutions')
            institutions.append({
                'curated_relation': record is not None,
                'name': a_value,
                'record': record,
            })
    else:
        for a_value in a_values:
            institutions.append({
                'curated_relation': False,
                'name': a_value,
            })

    return institutions


@jobs.over('description', '^520..')
def description(self, key, value):
    return value.get('a')


@jobs.over('position', '^245..')
def position(self, key, value):
    return value.get('a')


@jobs.over('ranks', '^656..')
def ranks(self, key, value):
    ranks = self.get('ranks', [])

    for value in force_list(value):
        for rank in force_list(value.get('a')):
            ranks.append(normalize_rank(rank))

    return ranks



# -*- coding: utf-8 -*-
#
# This file is part of INSPIRE.
# Copyright (C) 2014-2017 CERN.
#
# INSPIRE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# INSPIRE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with INSPIRE. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""DoJSON model definition for HEP."""

from __future__ import absolute_import, division, print_function

import itertools

import six

from inspire_schemas.utils import (
    convert_old_publication_info_to_new,
    normalize_arxiv_category,
)
from inspire_utils.helpers import force_list
from inspire_utils.record import get_value

from ..model import FilterOverdo, add_schema, clean_record


def add_arxiv_categories(record, blob):
    if not record.get('arxiv_eprints') or not blob.get('65017'):
        return record

    for category in force_list(get_value(blob, '65017')):
        if category.get('2') == 'arXiv' and category.get('a'):
            record['arxiv_eprints'][0]['categories'].append(
                normalize_arxiv_category(category['a'])
            )

    return record


def convert_publication_infos(record, blob):
    if not record.get('publication_info'):
        return record

    record['publication_info'] = convert_old_publication_info_to_new(record['publication_info'])

    return record


def move_incomplete_publication_infos(record, blob):
    def _keys_with_truthy_values(d):
        return {k for k, v in d.items() if v}

    publication_infos = record.get('publication_info', [])

    for i, publication_info in enumerate(publication_infos[:]):
        if _keys_with_truthy_values(publication_info).issubset({'journal_record', 'journal_title'}):
            public_note = {'value': u'Submitted to {}'.format(publication_info['journal_title'])}
            record.setdefault('public_notes', []).append(public_note)
            publication_infos.pop(i)

    return record


def ensure_document_type(record, blob):
    if not record.get('document_type'):
        record['document_type'] = ['article']

    return record


def ensure_curated(record, blob):
    if 'curated' not in record:
        record['curated'] = True

    return record


def convert_curated(record, blob):
    if blob.get('curated') is False:
        a_value = '* Temporary entry *' if blob.get('core') else '* Brief entry *'
        record.setdefault('500', []).insert(0, {'a': a_value})

    return record


def ensure_ordered_figures(record, blob):
    ordered_figures_dict = {}
    unordered_figures_list = []

    for figure in record.get('figures', []):
        order = figure.pop('order')
        if order:
            ordered_figures_dict[order] = figure
        else:
            unordered_figures_list.append(figure)

    record['figures'] = [value for key, value in sorted(six.iteritems(ordered_figures_dict))]
    record['figures'].extend(unordered_figures_list)
    return record


def ensure_unique_documents_and_figures(record, blob):
    def duplicates(elements):
        duplicate_keys_list = []
        for index, element in enumerate(elements):
            if element:
                if element['key'] in duplicate_keys_list:
                    yield index, element
                else:
                    duplicate_keys_list.append(element['key'])

    for index, attachment in itertools.chain(duplicates(record.get('documents', [])), duplicates(record.get('figures', []))):
        attachment['key'] = u'{}_{}'.format(index, attachment['key'])

    return record


def write_ids(record, blob):
    result_035 = record.get('035')
    id_dict = record.get('id_dict', {})

    for schema, values in six.iteritems(id_dict):
        z_values = iter(values)
        a_value = next(z_values)
        result_035.append({
            '9': schema,
            'a': a_value
        })
        for z_value in z_values:
            result_035.append({
                '9': schema,
                'z': z_value
            })

    if 'id_dict' in record:
        del record['id_dict']

    return record


def reorder_abstracts(record, blob):
    abstracts = record.get('abstracts', [])

    for index, abstract in enumerate(abstracts[:]):
        source = abstract.get('source') or ''
        if source.lower() == 'arxiv':
            abstracts.append(abstracts.pop(index))

    return record


hep_filters = [
    add_schema('hep.json'),
    add_arxiv_categories,
    convert_publication_infos,
    move_incomplete_publication_infos,
    reorder_abstracts,
    ensure_curated,
    ensure_document_type,
    ensure_unique_documents_and_figures,
    ensure_ordered_figures,
    clean_record,
]

hep2marc_filters = [
    write_ids,
    convert_curated,
    clean_record,
]

hep = FilterOverdo(filters=hep_filters)
hep2marc = FilterOverdo(filters=hep2marc_filters)
